from bzv.infra import Infra
from bzv.customerConfig import CustomerConfig
from bzv.configWatcher import ConfigWatcher
from bzv.inbox import Inbox
from bzv.server import main, optimize
from merescocore.components import readConfig
from bzv.observablehttpserver import BZVObservableHttpServer
from merescocore.framework import be
from weightless import Reactor
from sys import stdout, argv
from fcntl import flock, LOCK_EX, LOCK_NB
from os import environ, open, O_RDONLY, O_CREAT
from os.path import join as join_path
import sys, traceback
import syslog

environ['LANG']='en_US.UTF-8'

# Onderschep 'buitenste' exception en toon deze in syslog (zie: http://docs.python.org/library/sys.html)
def hooked(type, value, tb):
	syslog.syslog('meresco-server FATAL')
	dump = traceback.format_exception(type, value, tb)
	for  line in dump:
		syslog.syslog(line)

	# print ook naar stderror:
	traceback.print_exception(type, value, tb)

sys.excepthook = hooked;

args = argv[1:]
if len(args) == 1:
	configFile = args[0]
	config = readConfig(configFile)

# make sure no two instances access the same database
databasePath = config['databasePath']
lockFile = join_path(databasePath, 'lock')
fd = open(lockFile, O_RDONLY|O_CREAT)
flock(fd, LOCK_EX|LOCK_NB)

optimize(databasePath)

baseURL = config['baseURL']
portNumber = int(config['portNumber'])
sruUpdatePortNumber = int(config['sruUpdatePortNumber'])

# Infra needs a logfile at startup
loggingConfigfile = config['loggingConfig']
infra = Infra('Starting MerescoServer',loggingConfigfile=loggingConfigfile)

# Expect all config files here
configWatcherDirectory = config['configWatcherDirectory']
configfiles = config['configFiles'].split()


# Read all other configfiles and gather them in one configs dict
configs={}
for cfg in configfiles:
	configs[cfg] = CustomerConfig( configWatcherDirectory + '/' + cfg + '.cfg', infra )
infra.configs = configs

logging = infra.logging
logger = logging.getLogger('meresco-server')
logging.debug('Starting meresco-server logging')
logger.debug('Starting meresco-server logger')

print 'Firing up Tilburg Beter Zoeken & Vinden Server.'
reactor = Reactor()
observableHttpServer = BZVObservableHttpServer(reactor, portNumber)
observableHttpServer.startServer()
if portNumber < 1024:
	runAs(config['username'], config['groupname'])

sruUpdateServer = BZVObservableHttpServer(reactor, sruUpdatePortNumber, prio = 9 )

inbox = Inbox(reactor=reactor, inboxDirectory=config['inboxDirectory'],
	      doneDirectory=config['doneDirectory'], infra = infra)

configWatcher = ConfigWatcher(reactor=reactor,
			      configWatcherDirectory = config['configWatcherDirectory'],
			      infra = infra,
			      configs=configs,
			      )

server = be(main(reactor, observableHttpServer, sruUpdateServer, inbox,
		 baseURL, portNumber, databasePath, config, infra, configWatcher))

server.once.observer_init()

inbox.processInboxDirectory()

print "Ready to rumble on", baseURL, "at ports", portNumber, sruUpdatePortNumber
print "   - database:", databasePath, "\n"
stdout.flush()
reactor.loop()
